% The following Matlab code implements the GSL-decoder for MI decoding
% as described in "Covariation Informed Graph Slepians for Motor Imagery
% Decoding" by K Georgiadis,D A Adamos, S Nikolopoulos, N Laskaris
% and I Kompatsiaris

clear
%% Load data
load exemplar_data.mat
load sensor_info  
      
ref_STs=SMR_trials_train(:,:,sensor_kept_list);
[Ntrials,Ntime,Nsensors]=size(ref_STs);
kept_sensor_names=sensor_names(sensor_kept_list);kept_sensor_2D_coord=sensor_2D_coord(sensor_kept_list,:);
ref_STs_train=ref_STs;

%% Selectivity - Identify the sub-graph of interest using DS algorithm
% Here the identified sub-graph using the DS algorithm
% is provided. In order to run the DS algorithm described in "A graph 
% theoretic sensor selection scheme for covariance based Motor Imagery (MI)
% decoding" by K.Georgiadis et al line 19 must be uncommented and the 
% Dominant_graph_components folder must be added to the path

idx = [1,2,3,6,9,18,19,26,27,28,37,38,45,46,47,54,55,63,64,65,73];
% idx = dom_set(ref_STs_train,SMR_labels_train,Fs);

%% Graph Slepian Design
%Formulate W
spDISTs=squareform(pdist(kept_sensor_2D_coord));%spA=spDISTs<1*mean(spDISTs(:));
spA=-spDISTs;
A=exp(spA/1*mean(spDISTs(:)));
D = diag(sum(A));

CONST_NODES{1}=idx;
%define Bandwidth
CONST_W=50;

[basis,basis_eig0,basis_conc,basis_cut]=slepCompute(A,D,CONST_NODES,CONST_W,1);
SL0 = basis{2};

%% Apply to test data

ref_STs=SMR_trials_test(:,:,sensor_kept_list);
[Ntrials,Ntime,Nsensors]=size(ref_STs);
  
ref_STs_test=ref_STs; 

STs_ALL = [ref_STs_train;ref_STs_test];

%%% band-pass filtering 
[b,a]=butter(5,[8 30]/(Fs/2));
f_STs=[];for i_sensor=1:Nsensors,Trial=squeeze(STs_ALL(:,:,i_sensor));filtered_Trial=filtfilt(b,a,Trial')';f_STs(:,:,i_sensor)=filtered_Trial;end

FourierSTs_ALL=[];Energies=[];
for i=1:size(f_STs,1)
    ST = squeeze(f_STs(i,171+50:171+200+50,:))';
    FourierSTs = SL0'*ST; % eq(6)
    FourierSTs_ALL(i,:,:) = FourierSTs;
    trial_Energy=sum(abs(FourierSTs')); % eq(7)
    Energies(i,:)=trial_Energy;
end

FVs=[];FVs = Energies';

train_set=FVs(:,1:size(ref_STs_train,1));
test_set= FVs(:,size(ref_STs_train,1)+1:end);
train_labels=SMR_labels_train';
test_labels=SMR_labels_test';

[IDX, SCORE] = rankfeatures(train_set,train_labels,'criterion','wilcoxon');

% Identify the threshold for the feature selection process
threshold = rand_perm(train_set,train_labels,100);

k=numel(find(SCORE>threshold));
% Linear SVM for the classification task
Mdl = fitcsvm(train_set(IDX(1:k),:)',train_labels','Standardize',true,'KernelFunction','linear','KernelScale','auto');
[predictions,scores]=predict(Mdl,test_set(IDX(1:k),:)');
Classification_Error=numel(find(predictions-test_labels))/size(test_labels,1);
accuracy=1-Classification_Error;
